;;; otter.el --- major mode for editing and running Otter under Emacs
;; Revision 1.0
;; last edited on 1.4.1996

;; Copyright (C) 1996 Holger Schauer

;; Author: Holger Schauer <Holger.Schauer@gmd.de>
;; Keywords: modes theorem-proving

;;; This program is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 2 of the License, or
;;; (at your option) any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

;;; Commentary:

;; This package provides a major mode for editing input files
;; to the theorem prover Otter. It is also possible to "compile"
;; the whole buffer through otter and to display the output 
;; of otter.
;; Shortcuts for inserting lists are also provided.
;; It also knows about Otter's options and parameters - they're
;; available for insertion with minibuffer-completion.
;; Highlighting using font-lock is provided.
;; If you have easymenu installed this will be used.
;; This package was developed under XEmacs 19.13.
;; It is also known to work with XEmacs 19.11, FSF Emacs 19.29.1 and 19.30.


;;; Installation:

;; .. is easy as in most cases. Just add the following lines to your
;; .emacs and put otter-mode.el in a place where emacs can find it:
; append to auto-mode list - normally in your .emacs
;(setq auto-mode-alist
;      (append
;       '(("\\.ot$" . otter-mode)
;	 ("\\.in$" . otter-mode))
;       auto-mode-alist))
;(autoload 'otter-mode "otter-mode" "Mode for editing Otter files." t)
;(autoload 'run-otter "otter-mode" "Running an inferior Otter process." t)
;; 
;; You also need to tell where Otter resides for you 
;(defvar otter-program "<insert-absolute-path-to-otter>" "The 
; absolute path to Otter")



;;; TODO list:

;; enhance insertion so that it checks if a flag is already set or 
;; cleared etc.
;; add search for proofs and stops of search in the output
;; maybe otter-find-proof otter-find-sos otter-find-clauses
;; etc.

(provide 'otter-mode)
;; define some local variables
(defvar otter-program nil 
"*This must be set to the absolute path where otter will reside.")
(defvar otter-buffer "*otter*" "The name of the buffer in which otter runs.")
(defvar otter-out-extension ".oout" 
"*The extension used to generate output files from Otter.")
(defvar otter-in-extension1 "\\.ot"
"*The standard extension to recognize input-files for Otter. 
Note that the \\ before the period is a must !")
(defvar otter-in-extension2 "\\.in"
"*An other extension to recognize input-files for Otter.
Note that the \\ before the period is a must !")
(defvar otter-input-buffer nil
"*This points to the last used input-file buffer.")

;; append to auto-mode list - normally in your .emacs
(setq auto-mode-alist
      (append
       '(("\\.ot$" . otter-mode)
	 ("\\.in$" . otter-mode))
       auto-mode-alist))



;; now load the other parts of the mode
(require 'otter-init)
;(require 'otter-insert)
;(require 'otter-compile)


; finding things 
(defun otter-find-output-buffer ()
  "Switch to the last used output window of Otter."
  (interactive)
  (cond ((not (equal otter-buffer 'nil))
	 (switch-to-buffer otter-buffer)
	 (delete-other-windows))))

(defun otter-find-input-buffer ()
  "Switch to the last used input buffer."
  (interactive)
  (cond ((not (equal otter-buffer 'nil))
	 (switch-to-buffer otter-input-buffer)
	 (delete-other-windows))))

;; now set up the local key-bindings for the otter mode
(defvar otter-mode-map nil)
(defun otter-mode-commands (map)
  ; the shortcuts for inserting things
  (define-key map "\C-cif" 'otter-insert-formula-list)
  (define-key map "\C-cil" 'otter-insert-list)
  (define-key map "\C-ciw" 'otter-insert-weight-list)
  (define-key map "\C-cis" 'otter-insert-set-option)
  (define-key map "\C-cic" 'otter-insert-clear-option)
  (define-key map "\C-cia" 'otter-insert-parameter)
  (define-key map "\C-cii" 'otter-insert-include)
  (define-key map "\C-cim" 'otter-insert-me)
  (define-key map "\C-cio" 'otter-insert-op)
  (define-key map "\C-cix" 'otter-insert-lex)
  (define-key map "\C-cik" 'otter-insert-skolem)
  (define-key map "\C-cir" 'otter-insert-lrpo)
  ; compile 
  (define-key map "\C-c\C-cc" 'otter-compile-buffer)
  (define-key map "\C-c\C-cs" 'otter-compile-buffer-save)
  (define-key map "\C-c\C-cr" 'otter-compile-region)
  (define-key map "\C-c\C-cf" 'otter-compile-other-file)
  (define-key map "\C-c\C-co" 'otter-compile-other-file-save)
  ; miscellanous
  (define-key map "\C-cr" 'run-otter)
  ; finding things
  (define-key map "\C-fo" 'otter-find-output-buffer)
  (define-key map "\C-fi" 'otter-find-input-buffer)
;; the following bindings aren't used yet  
  (define-key map "\C-fp" 'otter-find-proof) 
  (define-key map "\C-fs" 'otter-find-sos)
  (define-key map "\C-fc" 'otter-find-clauses)
)

(if otter-mode-map
    nil
  (setq otter-mode-map (make-sparse-keymap))
  (otter-mode-commands otter-mode-map))


;; This sets up the maajor mode
(defun otter-mode ()
  "Major mode for editing OTTER input files.
  Several shortcuts and commands are provided. See C-hb for bindings."
  (interactive)
  (kill-all-local-variables)
  (use-local-map otter-mode-map)
  (setq mode-name "OTTER")
  (setq major-mode 'otter-mode)
  (make-local-variable 'comment-start)
  (setq comment-start "%")
  (run-hooks 'otter-mode-hook))


;; this is the simplest of all - just call otter-compile-other-file-save
;; this does all the rest. Needs perhaps a mode on it's own
;; if output-specific functionality gets added
(defun run-otter ()
  "Run Otter on a file and load/visit the input file. Automatically saves
   output of Otter."
  (interactive)
  (otter-compile-other-file-save))


;; if we're running XEmacs set colors for otter
(cond ((string-match "XEmacs\\|Lucid" emacs-version)
       (require 'font-lock)
       (add-hook 'otter-mode-hook      'turn-on-font-lock)
       (add-hook 'font-lock-mode-hook
		 '(lambda ()
		    (cond ((eq major-mode 'otter-mode)
			   (set (make-local-variable 'font-lock-keywords)
	           otter-font-lock-keywords))) ))
       (make-face 'font-lock-option-set-face)
       (set-face-background 'font-lock-option-set-face "WhiteSmoke")
       (set-face-foreground 'font-lock-option-set-face "red1")
       (make-face 'font-lock-option-clear-face)
       (set-face-background 'font-lock-option-clear-face "WhiteSmoke")
       (set-face-foreground 'font-lock-option-clear-face "green2")
;;       (set-face-background-pixmap 'font-lock-argument-face 
;;	"/usr/include/X11/bitmaps/gray")
       (defconst otter-font-lock-keywords
	 '(
	   ; Strings wie Strings :-)
	   ("\"[^\"]*\""  . font-lock-string-face)
	   ; Die verschiedenen Listen hervorheben
	   ("formula_list\(.*\)" . font-lock-function-name-face)
	   ("weight_list\(.*\)" . font-lock-function-name-face)
	   ("list\(.*\)" . font-lock-function-name-face)
	   ("end_of_list\." . font-lock-function-name-face)
	   ; Kommentare
	   ("\%.*" . font-lock-comment-face)
	   ; Otter-commandos
	   ("set" . font-lock-option-set-face)
	   ("clear" . font-lock-option-clear-face)
	   ("assign" . font-lock-option-set-face)
	   ("include" . font-lock-option-set-face)
	   ("op" . font-lock-option-set-face)
	   ("make_evaluable" . font-lock-option-set-face)
	   ("lex" . font-lock-option-set-face)
	   ("skolem" . font-lock-option-set-face)
	   ("lrpo" . font-lock-option-set-face)
	   ; Output
	   ("PROOF" . font-lock-option-set-face)
	   ("end of proof" . font-lock-option-set-face)
	   (".* clausifies to\:" . font-lock-string-face)
	   ("end of input processing" . font-lock-string-face)
	   ("start of search" . font-lock-string-face)
	   ("end of search" . font-lock-string-face)
	   ("Search stopped .*" . font-lock-option-set-face)
	   ("statistics" . font-lock-string-face)
	   ("times \(.*\)" . font-lock-string-face)
	   )) 
       ))

(defconst otter-menu
 '("Otter"
;   [ "Test" nil nil ]
   ("Compile" 
     [ "Buffer" otter-compile-buffer t ]
     [ "Buffer, saving output" otter-compile-buffer-save t ]
     [ "Other file" otter-compile-other-file t ]
     [ "Other file, saving output" otter-compile-other-file-save t ]
     [ "Region" otter-compile-region t])
   ("Insert"
    [ "set(flag)" otter-insert-set-option t ]
    [ "clear(flag)" otter-insert-clear-option t]
    [ "assign(par,value)" otter-insert-parameter t]
    [ "include(file)" otter-insert-include t]
    [ "operator" otter-insert-op t]
    [ "make_evaluable(sym,es)" otter-insert-me t]
    [ "lex(symbol_list)" otter-insert-lex t]
    [ "skolem(symbol_list)" otter-insert-skolem t]
    [ "lrpo" otter-insert-lrpo t]
    "--"
    [ "formula_list" otter-insert-formula-list t]
    [ "weight_list" otter-insert-weight-list t]
    [ "list (of clauses)" otter-insert-list t])
   ("Find"
    [ "Output-Buffer" otter-find-output-buffer t]
    [ "Recent Input-Buffer" otter-find-input-buffer t])
)
" Defines the menu structure for the otter mode.")

;; add a menu - using easymenu
(add-hook 'otter-mode-hook
 (lambda () 
   (cond ((featurep 'easymenu)       
       (easy-menu-define menubar-otter
	otter-mode-map 
	"Otter commands"
	otter-menu) 
       (cond ((string-match "XEmacs\\|Lucid" emacs-version)
	      (easy-menu-add otter-menu 'otter-mode-map)))))))

